#!/usr/bin/env python3

# procedure to get start and end times from Level-0 fpacket files for HARP2
#  The times are extracted from the detector image headers.  The detector number 
#  (1, 2, 3 or All) is also determined
#  Reference: HARP2 Telemetry Manual

__version__ = '1.0.0_2022-08-19'

from l0info_utils import read_packet, read_apid, get_anc_packet_time

def l0info_harp(args, fh, output):

    status = 0
    if args.verbose:
        print("Reading HARP2 science data file.")
        
    # Skip CFE header
    fh.seek(64)
    bSPW = True
    
    # Find first science fpacket (APID 751)
    apid = 0
    packetLength = 1
    while (apid != 751) and(packetLength > 0): 
        fpacket, packetLength = read_packet(fh, bSPW)
        apid = read_apid(fpacket)
    
    # If no science packets, rewind and get times from HKT packets (APID 750)
    if (packetLength==0):
        print("No science packets found in file")
        status = 110
        
        fh.seek(64)
        apid = 0
        packetLength = 1
        while (apid != 750) and (packetLength > 0):
            fpacket, packetLength = read_packet(fh, bSPW)
            apid = read_apid(fpacket)
        
        if fpacket:
            mpacket = fpacket
            try:
                stime = get_anc_packet_time(mpacket[6:12],0,'msec')
                print("start_time=%s" % stime.strftime('%Y-%m-%dT%H:%M:%S.%f'))
                if output:
                    output.write("start_time=%s\n" % stime.strftime('%Y-%m-%dT%H:%M:%S.%f'))
            except:
                return 120
            
            if fpacket:
                fpacket, packetLength = read_packet(fh, bSPW)
                apid = read_apid(fpacket)
                if apid == 750:  mpacket = fpacket
                
            etime = get_anc_packet_time(mpacket[6:12],0,'msec')
            
        return status
        
    # Get detector number from AP header
    detnum = (fpacket[12] & 48)>>4
      
      # Find an image header
    str_imhead = "HARP" # [72, 65, 82, 80]    # Header start mark ASCII "HARP"
    imhp = -1
    while (imhp == -1):
        str_packet = "".join([chr(x) for x in fpacket])
        if str_imhead in str_packet:
            imhp = str_packet.index(str_imhead)
        
        if imhp == -1: # If no header in current fpacket, find next science fpacket
            apid = 0
            packetLength = 1
            while (apid != 751) and (packetLength > 0):
                fpacket, packetLength = read_packet(fh, bSPW)
                apid = read_apid(fpacket)
          
            if packetLength==0:
                print('No image headers found in file.')                
                return 110
      
            # Check for more than one detector in file
            detn = (fpacket[12] & 48)>>4
            if detn != detnum: detnum = 999

    print("Image pointer: %d"%imhp)
    
    # Get start time
    try:
        stime = get_anc_packet_time(fpacket[imhp+7:imhp+13],0,'msec')
        print("start_time=%s" % stime.strftime('%Y-%m-%dT%H:%M:%S.%f'))
        if output:
            output.write("start_time=%s\n" % stime.strftime('%Y-%m-%dT%H:%M:%S.%f'))
    except:
        return 120

    # Read to end of file
    epacket = fpacket
    # Look for next image header
    packetLength = 1
    while packetLength>0:
        apid = 0

        # Find next science fpacket
        while (apid != 751) and (packetLength > 0):
            fpacket, packetLength = read_packet(fh, bSPW)
            apid = read_apid(fpacket)
        if fpacket:
            # Check for more than one detector in file
            detn = (fpacket[12] & 48)>>4
            if detn != detnum: detnum = 999
            # Look for image header
            imh = -1
            while (imh == -1) and fpacket:
                str_packet = "".join([chr(x) for x in fpacket])
                if str_imhead in str_packet:
                    imh = str_packet.index(str_imhead)
                
                if (imh == -1): # If no header in current fpacket, find next science fpacket
                    apid = 0
                    while (apid != 751) and (packetLength > 0):
                        fpacket, packetLength = read_packet(fh, bSPW)
                        apid = read_apid(fpacket)
            if fpacket:
                imhp = imh
                print("Image pointer: %d"%imhp)
                epacket = fpacket
    
    try:
        etime = get_anc_packet_time(epacket[imhp+7:imhp+13],0,'msec')
        print("stop_time=%s" % etime.strftime('%Y-%m-%dT%H:%M:%S.%f'))
        if output:
            output.write("stop_time=%s\n" % etime.strftime('%Y-%m-%dT%H:%M:%S.%f'))
    except:
        return 120
    
    if detnum == 999:
        print("detector=all")
        if output:
            output.write("detector=all\n")
    else:
        print("detector=%d"%detnum)
        if output:
            output.write("detector=%d\n"%detnum)
    
    return status

